/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.tartare;

import com.tartare.sandbox.ejb.JmsTest;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 *
 * @author oliechti
 */
public class JmsTestApplication {

    private final Logger LOGGER = Logger.getLogger(JmsTestApplication.class.getName());
    
    static ThreadLocal _ejb = new ThreadLocal();
    private Map<Integer, Integer> producedMessages = Collections.synchronizedMap(new HashMap<Integer, Integer>());
    private Map<Integer, Integer> consumedMessages = Collections.synchronizedMap(new HashMap<Integer, Integer>());

    public static JmsTest locateEjb() throws NamingException {
        if (_ejb.get() == null) {
            Context ctx = new InitialContext();
            _ejb.set(ctx.lookup("ejb/JmsTest"));
        }
        return (JmsTest) _ejb.get();
    }

    public void testSynchronous2() throws Exception {
        int k = 5;
        final CountDownLatch signal = new CountDownLatch(k);

        for (int i = 0; i < k; i++) {
            new Thread(new TestRunnable(signal, i)).start();
        }

        signal.await();
        System.out.println("All thread done");
        System.exit(0);
    }

    public class TestRunnable implements Runnable {

        private CountDownLatch latch;
        private int assignedGroup;

        public TestRunnable(CountDownLatch latch, int assignedGroup) {
            this.latch = latch;
            this.assignedGroup = assignedGroup;
        }

        public void run() {
            try {
                JmsTest c = locateEjb();

                for (int i = 0; i < 3000; i++) {


                    try {
                        System.out.println("Will send 10 messages  [id=" + Thread.currentThread().getName() + ", iteration #" + i + "]");

                        int k = 5;
                        while (k > 0) {
                            k--;
                            c.produceMessages(2, k);
                            Integer counter = producedMessages.get(k);
                            if (counter == null) {
                                counter = 0;
                            }
                            producedMessages.put(k, counter + 2);
                            System.out.println("Producing 2 messages in group " + k + ", " + Thread.currentThread().getName());
                        }

                        System.out.println("Will consume maximum 10 messages in group " + assignedGroup + " [id=" + Thread.currentThread().getName() + ",iteration #" + i + "]");
                        System.out.println("Iteration " + i);
                        System.out.println("Produced messages: " + producedMessages);
                        System.out.println("Consumed messages: " + consumedMessages);
                        c.consumeMessages(10, assignedGroup);
                        Integer counter = consumedMessages.get(assignedGroup);
                        if (counter == null) {
                            counter = 0;
                        }
                        consumedMessages.put(assignedGroup, counter + 10);
                        System.out.println("Done consuming 10 messages in group " + assignedGroup);
                    } catch (Exception e) {
                        System.out.println("Error in thread [id=" + Thread.currentThread().getName() + ",iteration #" + i + "]");
                        throw e;
                    }
                }

                LOGGER.log(Level.FINEST, "Finished thread");
                latch.countDown();
            } catch (Exception e) {
                LOGGER.log(Level.SEVERE, "Error in thread", e);
                latch.countDown();

            }
        }
    }

    public static void main(String[] args)
            throws Exception {
        new JmsTestApplication().testSynchronous2();
        System.exit(0);
    }
}
